import os
import cmd2


INTRO = """
Welcome to BasicShell!
======================
Please extend this class to create your own interactive shell applications.
Type 'help' to view the commands available in BasicShell.
"""

PROMPT = '(shell) '


class BasicShell(cmd2.Cmd):

    def __init__(self):
        super(BasicShell, self).__init__()
        self.debug = True
        self.intro = INTRO
        self.prompt = PROMPT
        self.current_dir = os.path.abspath(os.path.curdir)
        self.idx = -1
        self.results = {}
        self.current_key = None
        self.last_idx = self.idx

    def next_idx(self):
        self.idx += 1
        self.last_idx = self.idx
        return self.idx

    def add_result(self, data, desc=None):
        key = 'result_{}'.format(self.next_idx())
        self.results[key] = {'data': data, 'desc': desc}
        self.current_key = key

    ###

    def do_show_results(self, _=None):
        for k in self.results.keys():
            current = ''
            if k == self.current_key:
                current = '** current **'
            self.poutput('{}: {} {}'.format(k, self.results[k]['desc'], current))

    def do_set_result_data(self, line):
        items = [x.strip() for x in line.split('=')]
        self.results[items[0]]['data'] = items[1]

    def do_set_result_desc(self, line):
        items = [x.strip() for x in line.split('=')]
        self.results[items[0]]['desc'] = items[1]

    def do_set_current_key(self, key):
        self.current_key = key

    ###

    def do_cd(self, line):
        """ Usage: cd <dir path>
        Change the current directory to <dir path>
        """
        if line == '.' or line == '':
            self.do_pwd(None)
            return
        if line == '..':
            self.current_dir = os.path.split(self.current_dir)[0]
            self.do_pwd(None)
            return
        if not os.path.isdir(line):
            line = os.path.join(self.current_dir, line)
            if not os.path.isdir(line):
                self.poutput('Directory {} is not a valid directory'.format(line))
                return
        self.current_dir = line
        self.do_pwd(None)

    def do_ls(self, _=None):
        """ Usage: ls
        Lists the contents of the current directory. Same as shell command "!ls -lap".
        """
        self.do_shell('cd {}; ls -lap'.format(self.current_dir))

    def do_shell(self, line):
        """ Usage: !<command>
        Execute shell command <command>. For example, !echo $HOME will display the user's HOME directory.
        """
        if line is None or line is '':
            self.poutput('Please specify a shell command preceded by! For example, !echo $HOME')
        else:
            self.poutput('Running shell command: {}'.format(line))
            output = os.popen(line).read()
            self.poutput(output)

    def do_pwd(self, _=None):
        """ Usage: pwd
        Show the current directory.
        """
        self.poutput(self.current_dir)

    def do_undo(self, verbose=None):
        """ Usage: undo
        Move to the previous result set (if any).
        """
        idx = int(self.current_key.split('_')[1]) - 1
        idx = 0 if idx < 0 else idx
        self.idx = idx
        self.current_key = 'result_{}'.format(self.idx)
        if verbose:
            self.do_show_results(None)

    def do_redo(self, verbose=None):
        """ Usage: redo
        Move to the next result set (if any).
        """
        idx = int(self.current_key.split('_')[1]) + 1
        idx = self.last_idx if idx > self.last_idx else idx
        self.idx = idx
        self.current_key = 'result_{}'.format(self.idx)
        if verbose:
            self.do_show_results(None)

    @staticmethod
    def do_exit(_=None):
        """ Usage: exit
        Quits the shell application (same as quit).
        """
        return True


def main():
    import sys
    shell = BasicShell()
    sys.exit(shell.cmdloop())


if __name__ == '__main__':
    main()



import os
import unittest
import datetime

from types import SimpleNamespace


class MyException(Exception):
    pass


class MyTestCase(unittest.TestCase):

    def setup(self):
        pass

    def setUp(self):
        self.setup()

    def tear_down(self):
        pass

    def tearDown(self):
        self.tear_down()


class MyTestArguments(SimpleNamespace):
    pass


class Logger(object):

    def __init__(self, to_file=True, to_dir=None):
        self.f = None
        if to_file:
            now = datetime.datetime.now()
            file_name = 'log_{}.txt'.format(now.strftime('%Y%m%d_%H%M%S'))
            if to_dir:
                self.f = open(os.path.join(to_dir, file_name), 'w')
            else:
                self.f = open(file_name, 'w')

    def print(self, message):
        now = datetime.datetime.now()
        message = '[' + now.strftime('%Y-%m-%d %H:%M:%S.%f') + '] ' + str(message)
        print(message)
        if self.f:
            self.f.write(message + '\n')

    def __del__(self):
        self.f.close()



"""Top-level package for Barbell2."""

__author__ = """Ralph Brecheisen"""
__email__ = 'ralph.brecheisen@gmail.com'
__version__ = '1.14.0'

from .dicomexplorer.dicomexplorer import DicomExplorer
from .castorexportclient.castorexportclient import CastorExportClient



import os
import pydicom

from pydicom._dicom_dict import DicomDictionary
from barbell2.lib import BasicShell


class DicomExplorer(BasicShell):

    def __init__(self):
        super(DicomExplorer, self).__init__()
        self.intro = 'Welcome to DICOM Explorer!'
        self.prompt = '(dicom) '

    ###

    @staticmethod
    def is_dicom(file_path):
        if not os.path.isfile(file_path):
            return False
        try:
            with open(file_path, "rb") as f:
                return f.read(132).decode("ASCII")[-4:] == "DICM"
        except UnicodeDecodeError:
            return False

    @staticmethod
    def tag_for_name(name):
        for key, value in DicomDictionary.items():
            if name == value[4]:
                return hex(int(key))

    ###

    def do_load_file(self, file_path):
        """ Usage: load_file <file name or path>
        Load a single DICOM file. If only a file name is provided, it's location is assumed to be the
        current directory. Otherwise, the full path must be given.
        """
        if not os.path.isfile(file_path):
            file_path = os.path.join(self.current_dir, file_path)
        if not self.is_dicom(file_path):
            self.poutput('File is not DICOM')
            return
        self.poutput('Loading file...')
        self.add_result(file_path, desc='Single file')
        self.poutput('Ok')

    def do_load_dir(self, dir_path):
        """ Usage: load_dir <dir name or path>
        Load (recursively) all DICOM files in the given directory. If only the directory name is given, it is
        assumed that the directory is located in the current directory. Otherwise, the full path must be given.
        """
        if not os.path.isdir(dir_path):
            dir_path = os.path.join(self.current_dir, dir_path)
        self.poutput('Loading {}...'.format(dir_path))
        data = []
        for root, dirs, files in os.walk(dir_path):
            for f in files:
                if not f.startswith('._'):
                    f = os.path.join(root, f)
                    if self.is_dicom(f):
                        data.append(f)
                        self.poutput(f)
        self.add_result(data, desc='Files ({})'.format(len(data)))
        self.poutput('Loaded {} files'.format(len(data)))
        self.poutput('Ok')

    def do_show_files(self, n):
        """ Usage: show_files [n]
        Show files loaded in the current result set. If you want to show files from another result set
        select it first using the set_current_result command. If you specify n > 0, the first n files
        will be displayed. If you specify n < 0, the last n files will be displayed.
        """
        files = self.results[self.current_key]['data']
        try:
            n = 0 if n == '' else int(n)
        except ValueError as e:
            self.poutput(e)
        if n == 0:
            for f in files:
                self.poutput(f)
        elif n > 0:
            n = n if n < len(files) else len(files) - 1
            for i in range(n):
                self.poutput(files[i])
        elif n < 0:
            n = -n
            n = n if n < len(files) else len(files) - 1
            for i in range(len(files) - n, len(files)):
                self.poutput(files[i])
        else:
            self.poutput('Illegal n = {}'.format(n))
        self.poutput('Ok')

    def do_lookup_tag(self, tag_name):
        """ Usage: lookup_tag <tag name>
        Lookup tag <tag name> in the DICOM dictionary. You can specify only parts of a tag name, e.g.,
        "Transmit" will return multiple dictionary entries containing the word "Transmit" (like Transmit
        Coil Name). Being able to lookup tags comes in handy when you search for them in DICOM files.
        Note that (parts of) the tag name does not have to be case-sensitive.
        """
        for key, value in DicomDictionary.items():
            output = '{}: {}'.format(key, value)
            if tag_name == '':
                self.poutput(output)
            else:
                for item in value:
                    if tag_name in item:
                        self.poutput(output)
        self.poutput('Ok')

    def do_show_values(self, tag_name):
        """ Usage: show_values <tag name>
        Show values for tag <tag name> in the currently loaded DICOM files.
        """
        files = self.results[self.current_key]['data']
        tag = self.tag_for_name(tag_name)
        self.poutput(tag)
        for f in files:
            p = pydicom.read_file(f)
            if tag in list(p.keys()):
                self.poutput('{}: {}'.format(f, p[tag].value))
        self.poutput('Ok')

    def do_dump(self, file_path):
        """ Usage: dump file_path
        Dumps DICOM header for file <file_path>. If the file path does not exist, it must be only a file name
        and a search is done in the current result set files. If only one hit is found, its header will be
        dumped. If multiple hits have been found, a list of these file paths is shown from which the user
        must choose.
        """
        if os.path.isfile(file_path):
            p = pydicom.read_file(file_path)
            self.poutput(p)
        else:
            if self.current_key is None:
                self.poutput('No files loaded')
            else:
                files = self.results[self.current_key]['data']
                hits = []
                for f in files:
                    if file_path in f:
                        hits.append(f)
                if len(hits) == 0:
                    self.poutput('File {} not found'.format(file_path))
                elif len(hits) == 1:
                    p = pydicom.read_file(file_path)
                    self.poutput(p)
                else:
                    self.poutput('Choose one of the following candidates:')
                    for hit in hits:
                        self.poutput(hit)
        self.poutput('Ok')

    def do_check_pixels(self, verbose):
        """ Usage: check_pixels
        Checks for each file in the current result set whether its pixels can be loaded using the pydicom
        package. If not, this may be caused by the fact that the pixel values are compressed, e.g., in
        JPEG2000 format. To uncompress the values, use the 'to_raw' command of this tool.
        """
        files = self.results[self.current_key]['data']
        bad_files = []
        for f in files:
            p = pydicom.read_file(f)
            try:
                p.convert_pixel_data()
                if verbose != '':
                    self.poutput('OK: {}'.format(f))
            except NotImplementedError:
                bad_files.append(f)
        if len(bad_files) > 0:
            for f in bad_files:
                self.poutput('ERROR: {}'.format(f))
            self.add_result(bad_files, desc='Files with compressed pixels ({})'.format(len(bad_files)))
            self.do_show_results(None)
        self.poutput('Ok')

    def do_to_raw(self, output_dir):
        """ Usage: to_raw <output_dir>
        Decompress pixel values to raw format in case they have been compressed. This command requires that
        the tool gdcmconv is installed. To install it using HomeBrew (for Mac). The command requires an empty
        output directory where the converted DICOMs are saved.
        """
        result = os.system('which gdcmconv>/dev/null')
        if result > 0:
            self.poutput('Tool gdcmconv is not installed')
            return
        os.makedirs(output_dir, exist_ok=False)
        files = self.results[self.current_key]['data']
        for f in files:
            target_file = os.path.join(output_dir, os.path.split(f)[1])
            command = 'gdcmconv --raw {} {}'.format(f, target_file)
            os.system(command)
            self.poutput('Converted {}'.format(f))


def main():
    import sys
    shell = DicomExplorer()
    sys.exit(shell.cmdloop())


if __name__ == '__main__':
    main()



import os
import argparse
import h5py
import pydicom
import random

import numpy as np

from barbell2.utils import MyException, Logger
from barbell2.lib.dicom import Dcm2Numpy, Tag2NumPy

from sklearn.model_selection import train_test_split

LOG = None


def info_requested(args):
    if len(args) == 2 and args[1] == '--info':
        return True
    return False


def show_intro():
    LOG.print("""
    === CREATEH5 ===
    Welcome to the createh5 tool of the Barbell2 package!
    This tool allows you to create HDF5 files from collections of single DICOM images and associated TAG files that
    contain segmentations of various regions of interest in the DICOM images. You can use the tool to create training,
    validation and test sets for deep learning with the Keras framework.
    """)


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('root_dir', help='Root folder containing collection folders')
    parser.add_argument('output_dir', help='Output folder where train.h5, validation.h5 and test.h5 will be written')
    parser.add_argument('--output_file_name_training', help='Output file name training set', default='training.h5')
    parser.add_argument('--output_file_name_validation', help='Output file name validation set', default='validation.h5')
    parser.add_argument('--output_file_name_testing', help='Output file name test set', default='testing.h5')
    parser.add_argument('--height', type=int, help='Image height in pixels (rows in DICOM header)', default=512)
    parser.add_argument('--width', type=int, help='Image width in pixels (columns in DICOM header)', default=512)
    parser.add_argument('--training', help='Comma-separated list of collection names for training')
    parser.add_argument('--validation', help='Comma-separated list of collection names for validation')
    parser.add_argument('--testing', help='Comma-separated list of collection names for validation')
    parser.add_argument('--split', help='Comma-separated list of collection names for a 80/20 split between training and validation')
    parser.add_argument('--split_percentage', type=float, help='Percentage to split on (default: .8)', default=0.8)
    # https://stackoverflow.com/questions/8259001/python-argparse-command-line-flags-without-arguments
    # parser.add_argument('--info', help='Shows detailed help info', action='store_true')
    args = parser.parse_args()
    return args


def check_collections(root_dir, collections):
    collection_list = [x.strip() for x in collections.split(',')]
    for collection in collection_list:
        if not os.path.isdir(os.path.join(root_dir, collection)):
            raise MyException('Collection directory "{}" does not exist'.format(collection))
    LOG.print('Done')


def has_correct_dimensions(dcm_file, width, height):
    p = pydicom.read_file(dcm_file)
    if int(p.Rows) == height and int(p.Columns) == width:
        return True
    return False


def collect_files(root_dir, collections, width, height):
    f_skip = open('skipped.txt', 'a')
    file_list = []
    collection_list = [x.strip() for x in collections.split(',')]
    LOG.print(collection_list)
    for collection in collection_list:
        collection_dir = os.path.join(root_dir, collection)
        for root, dirs, files in os.walk(collection_dir):
            for f in files:
                if f.endswith('.dcm') and not f.startswith('._'):
                    dcm_file = os.path.join(root, f)
                    # Only allow images to be included that have the correct dimension (e.g., 512x512)
                    if has_correct_dimensions(dcm_file, width, height):
                        tag_file = os.path.join(root, f)[:-4] + '.tag'
                        if os.path.isfile(tag_file):
                            file_list.append([dcm_file, tag_file])
                            LOG.print('Adding {} to collection'.format(dcm_file))
                        else:
                            f_skip.write('missing TAG file: {}\n'.format(dcm_file))
                    else:
                        f_skip.write('wrong dimensions: {}\n'.format(dcm_file))
    f_skip.close()
    return file_list


def shuffle_file_list(file_list):
    random.shuffle(file_list)
    return file_list


def split_file_list(file_list, percentage):
    x_train, x_test = train_test_split(file_list, test_size=1.0 - percentage)
    return x_train, x_test


def get_dcm_pixels(file_path):
    dcm2numpy = Dcm2Numpy()
    dcm2numpy.set_input_dicom_file_path(file_path)
    dcm2numpy.set_normalize_enabled()
    dcm2numpy.execute()
    return dcm2numpy.get_output_numpy_array()


def get_tag_pixels(file_path, shape):
    tag2numpy = Tag2NumPy(shape)
    tag2numpy.set_input_tag_file_path(file_path)
    tag2numpy.execute()
    pixels = tag2numpy.get_output_numpy_array()
    # Note that sometimes pixels = None because of failure to reshape the pixel array
    # to the size of the DICOM image
    return pixels


def update_labels(pixels):
    # Alberta protocol: AIR = 0, MUSCLE = 1, IMAT = 2, VAT = 5, SAT = 7
    # pixels[pixels == 0] = 0
    pixels[pixels == 2] = 0
    pixels[pixels == 4] = 0
    pixels[pixels == 11] = 0
    pixels[pixels == 12] = 0
    pixels[pixels == 14] = 0
    # pixels[pixels == 1] = 1
    pixels[pixels == 5] = 2
    pixels[pixels == 7] = 3
    return pixels


def labels_ok(labels):
    for label in [0, 1, 2, 3]:
        if label not in labels:
            return False
    return len(labels) == 4


def update_label_counts(label_counts, labels):
    for label in labels:
        if label not in label_counts.keys():
            label_counts[label] = 1
        else:
            label_counts[label] += 1
    return label_counts


def create_h5_from_file_list(file_list, output_file_path):
    f_skip = open('skipped.txt', 'a')
    label_counts = {}
    with h5py.File(output_file_path, 'w') as h5f:
        count = 1
        for file_pair in file_list:
            file_name = os.path.split(file_pair[0])[1][:-4]
            dcm_pixels = get_dcm_pixels(file_pair[0])
            tag_pixels = get_tag_pixels(file_pair[1], dcm_pixels.shape)
            if tag_pixels is None:
                LOG.print('ERROR: Could not retrieve pixels from {}'.format(file_pair[1]))
                f_skip.write('error retrieving TAG pixels: {}\n'.format(file_pair[0]))
                continue
            tag_pixels = update_labels(tag_pixels)
            labels = np.unique(tag_pixels)
            if not labels_ok(labels):
                LOG.print('ERROR: Labels not ok ({})'.format(labels))
                f_skip.write('wrong labels TAG file: {}\n'.format(file_pair[0]))
                continue
            label_counts = update_label_counts(label_counts, labels)
            group = h5f.create_group('{}'.format(file_name))
            group.create_dataset('images', data=dcm_pixels)
            group.create_dataset('labels', data=tag_pixels)
            LOG.print('{:04d} added images and labels ({}) for patient {}'.format(count, labels, file_name))
            count += 1
    LOG.print('Done')
    LOG.print('{}: {}'.format(output_file_path, label_counts))
    f_skip.close()


def run(args):

    # Verify that root folder exists and is not empty
    if not os.path.isdir(args.root_dir):
        raise MyException('Root directory "{}" does not exist'.format(args.root_dir))
    if len(os.listdir(args.root_dir)) == 0:
        raise MyException('Root directory "{}" is empty'.format(args.root_dir))

    # Verify that training, validation and test collections exist and are not empty
    if args.training is not None:
        LOG.print('Checking collections training...')
        check_collections(args.root_dir, args.training)
    if args.validation is not None:
        check_collections(args.root_dir, args.validation)
    if args.testing is not None:
        check_collections(args.root_dir, args.testing)
    if args.split is not None:
        check_collections(args.root_dir, args.split)
        if args.split_percentage is None:
            raise MyException('Argument --split_percentage is mandatory when choosing --split')

    # Create training H5
    if args.training is not None:
        file_list = collect_files(
            args.root_dir, args.training, args.width, args.height)
        file_list = shuffle_file_list(file_list)
        create_h5_from_file_list(
            file_list,
            os.path.join(args.output_dir, args.output_file_name_training))

    # Create validation H5
    if args.validation is not None:
        file_list = collect_files(
            args.root_dir, args.validation, args.width, args.height)
        file_list = shuffle_file_list(file_list)
        create_h5_from_file_list(
            file_list,
            os.path.join(args.output_dir, args.output_file_name_validation))

    # Create test H5
    if args.testing is not None:
        file_list = collect_files(
            args.root_dir, args.testing, args.width, args.height)
        file_list = shuffle_file_list(file_list)
        create_h5_from_file_list(
            file_list,
            os.path.join(args.output_dir, args.output_file_name_testing))

    # Create percentage split training/validation. Note that the split is done
    # across all collections, so everything is considered to be one big data set
    # that is split into parts.
    # Note: the args.split_percentage refers to the percentage of observations
    # assigned to the training set!
    if args.split is not None:
        file_list = collect_files(args.root_dir, args.split, args.width, args.height)
        file_list = shuffle_file_list(file_list)
        training_files, validation_files = split_file_list(file_list, args.split_percentage)
        LOG.print('>>> Creating training.h5...')
        create_h5_from_file_list(training_files, os.path.join(args.output_dir, args.output_file_name_training))
        LOG.print('>>> Creating validation.h5...')
        create_h5_from_file_list(validation_files, os.path.join(args.output_dir, args.output_file_name_validation))


def main():
    global LOG
    args = get_args()
    os.makedirs(args.output_dir, exist_ok=False)
    LOG = Logger(args.output_dir)
    show_intro()
    try:
        run(args)
    except MyException as e:
        LOG.print(e)


if __name__ == '__main__':
    main()



import os
import json
import numpy as np
import SimpleITK as sitk
from radiomics import featureextractor


class CalculateFeatures(object):

    def __init__(self):
        self._input_image_file_path = None
        self._input_mask_file_path = None
        self._wavelet_enabled = False
        self._log_enabled = False
        self._log_custom_args = None
        self._2d = True
        self._3d = False
        self._output_dir = None
        self._settings = {}
        self._settings_file_path = None
        self._apply_slope_and_intercept = True
        self._output_file_path = None

    # INTERFACE

    def set_input_image_file_path(self, file_path):
        self._input_image_file_path = file_path

    def set_input_mask_file_path(self, file_path):
        self._input_mask_file_path = file_path

    def set_wavelet_enabled(self, value):
        self._wavelet_enabled = value

    def set_log_enabled(self, value):
        if value and self._2d:
            raise RuntimeError('Set calculation to 3D to allow Log of Gaussian (LoG) features')
        self._log_enabled = value

    def set_log_custom_args(self, custom_args):
        if self._2d:
            raise RuntimeError('Set calculation to 3D to allow Log of Gaussian (LoG) features')
        self._log_custom_args = custom_args

    def apply_default_log_custom_args(self):
        self.set_log_custom_args({'sigma': [1.0, 2.0, 3.0, 4.0, 5.0]})

    def set_2d(self):
        if self._log_enabled:
            raise RuntimeError('Disable Log of Gaussian (LoG) features before calculating 2D radiomics')
        self._2d = True
        self._3d = False
        self.set_setting('force2D', True)

    def set_3d(self):
        self._2d = False
        self._3d = True
        self.set_setting('force2D', False)

    def set_output_dir(self, output_dir):
        self._output_dir = output_dir

    def set_settings(self, settings):
        self._settings = settings

    def set_settings_file_path(self, file_path):
        self._settings_file_path = file_path

    def set_setting(self, name, value):
        self._settings[name] = value

    def set_apply_slope_and_intercept_enabled(self, value):
        self._apply_slope_and_intercept = value

    def apply_default_settings(self):
        self.set_setting('interpolator', sitk.sitkBSpline)
        self.set_setting('resampledPixelSpacing', None)
        self.set_setting('padDistance', 10)
        self.set_setting('resegmentRange', [-3, 3])
        self.set_setting('resegmentMode', 'sigma')
        self.set_setting('voxelArrayShift', 1000)
        self.set_setting('label', 1)

    def get_output_file_path(self):
        return self._output_file_path

    # INTERNAL METHODS

    @staticmethod
    def match_orientations(image, mask, file_name):
        # This function checks whether the sign of the orientations of image and mask match with
        # each other. If not, the directions are flipped in the mask.
        img_dir = image.GetDirection()
        msk_dir = mask.GetDirection()
        new_dir = list(msk_dir)
        for i in range(len(img_dir)):
            x, y = np.sign(img_dir[i]), np.sign(msk_dir[i])
            if x != y:
                print('Mismatching orientation image and mask: {} <> {}'.format(x, y))
                new_dir[i] = -1.0 * msk_dir[i]
        mask.SetDirection(tuple(new_dir))
        return image, mask

    def _load_image_and_mask_files(self, image_file, mask_file):
        image = sitk.ReadImage(image_file)
        mask = sitk.ReadImage(mask_file)
        # Check that orientation of image and mask is the same and correct it if not
        image, mask = self.match_orientations(image, mask, file_name=os.path.split(image_file)[1])
        return image, mask

    @staticmethod
    def _write_features(features, file_path):
        with open(file_path, 'w') as f:
            for k in features.keys():
                f.write(k + ' = ' + str(features[k]))
                f.write('\n')
        return file_path

    # EXECUTE

    def execute(self, verbose=False):
        image, mask = self._load_image_and_mask_files(self._input_image_file_path, self._input_mask_file_path)
        if self._settings_file_path:
            extractor = featureextractor.RadiomicsFeatureExtractor(self._settings_file_path)
        else:
            extractor = featureextractor.RadiomicsFeatureExtractor(**self._settings)
        extractor.enableImageTypeByName('Original')
        if self._wavelet_enabled:
            extractor.enableImageTypeByName('Wavelet')
        extractor.enableAllFeatures()
        if self._log_enabled:
            extractor.enableImageTypeByName('LoG', customArgs=self._log_custom_args)
        features = extractor.execute(image, mask)
        os.makedirs(self._output_dir, exist_ok=True)
        if verbose:
            print(json.dumps(features, indent=4))
        file_name = os.path.split(self._input_mask_file_path)[1]
        if file_name.endswith('.nii.gz'):
            file_name = file_name[:-7] + '.txt'
            file_path = os.path.join(self._output_dir, file_name)
            file_path = self._write_features(features, file_path)
            self._output_file_path = file_path
            if verbose:
                print(file_path)
        elif file_name.endswith('.dcm'):
            file_name = file_name[:-4] + '.txt'
            file_path = os.path.join(self._output_dir, file_name)
            file_path = self._write_features(features, file_path)
            self._output_file_path = file_path
            if verbose:
                print(file_path)
        else:
            raise RuntimeError('Unsupported mask file extension {}'.format(
                os.path.splitext(self._input_mask_file_path)[1]))


if __name__ == '__main__':
    node = CalculateFeatures()
    node.set_2d()
    node.set_input_image_file_path('/Volumes/USB_SECURE1/data/radiomics/projects/004_ovarium/data/test/out/IM_1.nii.gz')
    node.set_input_mask_file_path('/Volumes/USB_SECURE1/data/radiomics/projects/004_ovarium/data/test/out/IM_1_tag_muscle.nii.gz')
    node.set_output_dir('/Volumes/USB_SECURE1/data/radiomics/projects/004_ovarium/data/test/out')
    node.apply_default_settings()
    node.execute()
    print(node.get_output_file_path())
